package csir.helloworld.seam;

import java.io.Serializable;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import javax.interceptor.Interceptors;
import javax.ejb.Stateless;
import javax.faces.application.FacesMessage;
import javax.faces.context.FacesContext;
import javax.persistence.EntityManager;

import org.apache.log4j.Logger;
import org.hibernate.validator.Valid;
import org.jboss.seam.Seam;
import org.jboss.seam.annotations.IfInvalid;
import org.jboss.seam.annotations.In;
import org.jboss.seam.core.FacesMessages;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Out;
import org.jboss.seam.annotations.Outcome;
import org.jboss.seam.ejb.SeamInterceptor;

import csir.helloworld.par.HelloWorldEntity;

/**
 * 
 * Stateless bean to create a new entity bean and persist it
 * @author  <a href="mailto:louis.coetzee@csir.co.za">Louis Coetzee</a>
 *
 */
@Stateless
@Name("helloWorldBean")
public class HelloWorldBean implements HelloWorld, Serializable {
	/**
	 * nothing
	 */
	private static final long serialVersionUID = 1500425901017997440L;

	static final Logger logger = Logger.getLogger(HelloWorldBean.class);

	 @In(create = true, value="jaasExampleEntityManager")
	 EntityManager em;
	 
	/**
	 * Force Hibernate validatoin on those elements having constraints in Entity
	 * bean other are ignored
	 */
	@In(create = true,required=true)
	@Out(required=false)
	@Valid
	private HelloWorldEntity helloWorldEntity;

  @In(create = true, required = true)
        private transient FacesMessages facesMessages;


	
	public HelloWorldBean() {
		logger.info("HelloWorldBean started");
	}

	@IfInvalid(outcome = Outcome.REDISPLAY)
	public String createHelloWorldMessageAction() {
		
		logger.info("User creating helloWord:  " + FacesContext.getCurrentInstance().getExternalContext().getUserPrincipal().getName().toString());		
		logger.info("Suppose to persist message: " + " "
				+ helloWorldEntity);
		try {
			em.persist(helloWorldEntity);
			em.flush();
		} catch (Exception e) {
			logger.error(e.getMessage());
			FacesMessage facesMessage = new FacesMessage(e.getMessage());
			facesMessages.add(facesMessage);
			return null;
		}
		return "success";
	}
}
