package csir.user.par;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.Table;
import javax.persistence.Transient;
import javax.persistence.Version;

import org.apache.log4j.Logger;
import org.hibernate.validator.Length;
import org.hibernate.validator.NotNull;
import org.jboss.seam.ScopeType;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Role;
import org.jboss.seam.annotations.Scope;


/**
 * Entity bean containing the portalUser with a relationship to its roles @see PortalRole
 * 
 * @author <a href="mailto:louis.coetzee@csir.co.za">Louis Coetzee</a>
 *
 */
@Entity
@Name("portalUser")
@Scope(ScopeType.SESSION)
@Table(name = "portal_user")
public class PortalUser implements Serializable {

	private static final long serialVersionUID = -8582750686315388835L;
	static final Logger logger = Logger.getLogger(PortalUser.class);

	private Integer id;

	private Integer version;

	private String login;

	private String password;
	
	private String transientPassword1;
	private String transientPassword2;
	
	private String title;
	private String firstname;
	private String surname;
	private String email;
	private String telephoneNumber;
	private String cellNumber;
	private String organisation;


	private Boolean active;

	private List<PortalRole> roles;
	

	public PortalUser() {
		logger.info("**************** PortalUser construct ********");
		id = 0;
		roles = new ArrayList<PortalRole>();
	}

	public Boolean getActive() {
		return active;
	}

	public void setActive(Boolean active) {
		this.active = active;
	}

	@NotNull
	@Length(min=1, max=50, message="Name must have length between 1 and 50 characters")
	public String getFirstname() {
		return firstname;
	}

	public void setFirstname(String firstname) {
		this.firstname = firstname;
	}

	@NotNull
	@Length(min=1, max=10, message="Title must have length between 1 and 10 characters")
	public String getTitle() {
		return title;
	}

	public void setTitle(String title) {
		this.title = title;
	}

	@NotNull
	@Length(min=1, max=100, message="Email must have length between 1 and 100 characters")
	public String getEmail() {
		return email;
	}

	public void setEmail(String email) {
		this.email = email;
	}

	@Length(min=0, max=50, message="Telephone Number must have length between 0 and 50 characters")
	public String getTelephoneNumber() {
		return telephoneNumber;
	}

	public void setTelephoneNumber(String telephoneNumber) {
		this.telephoneNumber = telephoneNumber;
	}

	@Length(min=0, max=50, message="Cell Number must have length between 0 and 50 characters")
	public String getCellNumber() {
		return cellNumber;
	}

	public void setCellNumber(String cellNumber) {
		this.cellNumber = cellNumber;
	}

	@Length(min=0, max=50, message="Organisation must have length between 0 and 50 characters")
	public String getOrganisation() {
		return organisation;
	}

	public void setOrganisation(String organisation) {
		this.organisation = organisation;
	}

	@Id  @GeneratedValue(strategy=GenerationType.AUTO)
	@Column(name = "portaluser_id")
	public Integer getId() {
		return id;
	}

	public void setId(Integer id) {
		this.id = id;
	}

	@NotNull
	@Length(min=5, max=8, message="Username must have length between 5 and 8 characters")
	public String getLogin() {
		return login;
	}

	public void setLogin(String login) {
		this.login = login;
	}

	public String getPassword() {
		return password;
	}

	public void setPassword(String password) {
		this.password = password;
	}

	@Transient
	public String getTransientPassword1() {
		return transientPassword1;
	}

	public void setTransientPassword1(String transientPassword1) {
		this.transientPassword1 = transientPassword1;
	}

	@Transient
	public String getTransientPassword2() {
		return transientPassword2;
	}

	public void setTransientPassword2(String transientPassword2) {
		this.transientPassword2 = transientPassword2;
	}

	@NotNull
	@Length(min=1, max=50, message="Surname must have length between 1 and 50 characters")	
	public String getSurname() {
		return surname;
	}

	public void setSurname(String surname) {
		this.surname = surname;
	}

	@Version
	public Integer getVersion() {
		return version;
	}

	public void setVersion(Integer version) {
		this.version = version;
	}
 
	@ManyToMany(cascade = { CascadeType.MERGE, CascadeType.PERSIST,
			CascadeType.REFRESH },targetEntity=PortalRole.class)
	@JoinTable(name = "portaluser_portalrole", joinColumns = @JoinColumn(name = "portaluser_id"), inverseJoinColumns = @JoinColumn(name = "portalrole_id"))
	public List<PortalRole> getRoles() {
		return roles;
	}

	public void setRoles(List<PortalRole> roles) {
		this.roles = roles;
	}
	

	public String toString() {
		StringBuilder sb = new StringBuilder();
		sb.append(id + " : ");
		sb.append(login);
		sb.append(", ");
		sb.append(firstname);
		sb.append(", ");
		sb.append(surname + "(Roles: ");
		if (null != roles) {
			for (PortalRole r : roles) {
				sb.append(r.toString() + "; ");
			}
		}
		sb.append(")");

		return sb.toString();
	}
}
